#include <time.h>
#include <stdio.h>
#include <stdlib.h>

void rnd_init(int *array, int size) {
	int i;

	srand(time(NULL));
	for (i = 0; i < size; i++)
		array[i] = rand();
}

void dump(int *array, int size) {
	int i;

	for (i = 0; i < size; i++)
		printf("%2d. <%5d>\n", i, array[i]);
}

/*
void swap(int *a, int *b) {
	int t;

	t = *a;
	*a = *b;
	*b = t;
}
*/

/*
	The calling conventions are explained here:
	https://en.wikipedia.org/wiki/X86_calling_conventions#x86-64_calling_conventions
*/

#if defined(__x86_64)
#pragma message("Code branch: x86_64, 64-bit calling convention, no decorators")
extern void swap(int *a, int *b);
#elif defined(_MSC_VER)
#pragma message("Code branch: MSVC, 32-bit fastcall calling convention, decorators @<fname>@<argsize>")
extern void _fastcall swap(int *a, int *b);
#elif defined(__GNUC__) || defined(__MINGW32__)
#pragma message("Code branch: GCC, 32-bit fastcall calling convention, decorators @<fname>@<argsize>")
extern void swap(int *a, int *b) __attribute__((fastcall));
#endif

void quicksort(int *array, int left, int right) {
  int li = left, ri = right;
  int pivot;

  /* Pivot Selection */
  pivot = array[(left + right) / 2];

  /* Partitioning */
  do {
	  while (li <= right && array[li] < pivot) li++;
	  while (ri >= left && array[ri] > pivot) ri--;
	  if (li <= ri) {
		  if (li != ri) swap(&array[li], &array[ri]);
		  li++;
		  ri--;
	  }
  } while (li <= ri);

  /* Recursion */
  if (left < ri) quicksort(array, left, ri);
  if (right > li) quicksort(array, li, right);
}

int main() {
	int data[20];

	/*
	int a = 5, b = 7;
	printf("%d %d\n", a, b);
	swap(&a, &b);
	printf("%d %d\n", a, b);
	*/

	rnd_init(data, 20);

	printf("Random init:\n");
	dump(data, 20);

	quicksort(data, 0, 19);

	printf("\nQuick sorted:\n");
	dump(data, 20);

	return 0;
}
